import {
  cloneObject,
  renderMessages,
  toValidFilename
} from "./chunk-YPQCEOVC.js";
import {
  Logger,
  __name,
  __require,
  formatMilliseconds,
  getTimestamp,
  useLevelFilter,
  useNamespaceFilter
} from "./chunk-G3LBBVZT.js";

// src/node/clipboard.ts
function pbcopy(data) {
  return new Promise(function(resolve5, reject) {
    const proc = __require("child_process").spawn("pbcopy");
    proc.on("error", (err) => reject(err));
    proc.on("close", (err) => resolve5(data));
    proc.stdin.write(data);
    proc.stdin.end();
  });
}
__name(pbcopy, "pbcopy");

// src/node/env.ts
import fs from "fs";
import { resolve } from "path";
var log = Logger("zeed:env");
var NEWLINE = "\n";
var RE_INI_KEY_VAL = /^\s*([\w_.-]+)\s*=\s*(.*)?\s*$/;
var RE_NEWLINES = /\\n/g;
var NEWLINES_MATCH = /\n|\r|\r\n/;
function parse(src, options = {}) {
  let obj = {};
  src.toString().split(NEWLINES_MATCH).forEach(function(line, idx) {
    const keyValueArr = line.match(RE_INI_KEY_VAL);
    if (keyValueArr != null) {
      const key = keyValueArr[1];
      let val = keyValueArr[2] || "";
      const end = val.length - 1;
      const isDoubleQuoted = val[0] === '"' && val[end] === '"';
      const isSingleQuoted = val[0] === "'" && val[end] === "'";
      if (isSingleQuoted || isDoubleQuoted) {
        val = val.substring(1, end);
        if (isDoubleQuoted) {
          val = val.replace(RE_NEWLINES, NEWLINE);
        }
      } else {
        val = val.trim();
      }
      obj[key] = val;
    } else {
      log.debug(`did not match key and value when parsing line ${idx + 1}: ${line}`);
    }
  });
  return obj;
}
__name(parse, "parse");
function stringToPath(value, defaultValue = ".") {
  return resolve(process.cwd(), value != null ? value : defaultValue);
}
__name(stringToPath, "stringToPath");
function valueToPath(value, defaultValue = "") {
  if (value == null)
    value = defaultValue;
  return stringToPath(String(value).trim(), defaultValue);
}
__name(valueToPath, "valueToPath");
var toPath = valueToPath;
function setupEnv(options = {}) {
  var _a2, _b2, _c, _d;
  const dotenvPath = (_b2 = options == null ? void 0 : options.path) != null ? _b2 : toPath((_a2 = options == null ? void 0 : options.filename) != null ? _a2 : ".env");
  const encoding = (_c = options == null ? void 0 : options.encoding) != null ? _c : "utf8";
  const debug = (options == null ? void 0 : options.debug) || false;
  if (debug !== true)
    log.level = Infinity /* off */;
  try {
    const parsedEnv = fs.existsSync(dotenvPath) ? parse(fs.readFileSync(dotenvPath, { encoding }), { debug }) : {};
    const parsedEnvLocal = fs.existsSync(dotenvPath + ".local") ? parse(fs.readFileSync(dotenvPath + ".local", { encoding }), { debug }) : {};
    const parsed = Object.assign({}, parsedEnv, parsedEnvLocal);
    let env = (_d = options == null ? void 0 : options.env) != null ? _d : process.env;
    Object.entries(parsed).forEach(([key, value]) => {
      if (typeof (options == null ? void 0 : options.prefix) === "string") {
        key = (options == null ? void 0 : options.prefix) + key;
      }
      if (!Object.prototype.hasOwnProperty.call(env, key)) {
        if (value != null) {
          log.info(`set env.${key} = ${value}`);
          env[key] = value;
        }
      } else {
        log.debug(`"${key}" is already defined and will not be overwritten`);
      }
    });
    return { parsed };
  } catch (e) {
    log.error(e);
    return { error: e };
  }
}
__name(setupEnv, "setupEnv");

// src/node/filestorage.ts
import {
  mkdirSync,
  readdirSync,
  readFileSync,
  rmSync,
  unlinkSync,
  writeFileSync
} from "fs";
import { dirname, resolve as resolve2 } from "path";
var log2 = Logger("zeed:filestorage");
var FileStorage = class {
  constructor(opt = {}) {
    this.store = {};
    this.fileKeys = void 0;
    this.pretty = false;
    var _a2, _b2, _c, _d;
    this.dirname = resolve2(process.cwd(), opt.path || ".fileStorage");
    this.pretty = !!opt.pretty;
    this.extension = (_a2 = opt.extension) != null ? _a2 : ".json";
    if (opt.extension && !this.extension.startsWith(".")) {
      this.extension = "." + this.extension;
    }
    this.extensionLength = this.extension.length;
    this.objectToString = (_b2 = opt.objectToString) != null ? _b2 : (data) => {
      return this.pretty ? JSON.stringify(data, null, 2) : JSON.stringify(data);
    };
    this.objectFromString = (_c = opt.objectFromString) != null ? _c : (data) => {
      try {
        return JSON.parse(data);
      } catch (err) {
        log2.warn(`fileStorage parse error '${err}' in`, data);
      }
    };
    this.keyToFilename = (_d = opt.keyToFilename) != null ? _d : toValidFilename;
  }
  setItem(key, value) {
    this.store[key] = cloneObject(value);
    try {
      const data = this.objectToString(value);
      const path = this.getPath(key);
      mkdirSync(dirname(path), { recursive: true });
      writeFileSync(path, data, "utf8");
    } catch (err) {
      log2.error("setItem error", err);
    }
  }
  getPath(key) {
    return resolve2(this.dirname, this.keyToFilename(key) + this.extension);
  }
  getBuffer(key) {
    const path = this.getPath(key);
    return Buffer.from(readFileSync(path));
  }
  getItem(key) {
    let value = this.store[key];
    if (value === null)
      return;
    if (value != null) {
      return cloneObject(value);
    }
    try {
      const path = this.getPath(key);
      const data = readFileSync(path, "utf8");
      if (data != null) {
        const value2 = this.objectFromString(data);
        this.store[key] = value2;
        return value2;
      }
    } catch (err) {
      log2.warn("getItem error", err);
      this.store[key] = null;
    }
  }
  removeItem(key) {
    delete this.store[key];
    if (this.fileKeys != null) {
      const index = this.fileKeys.indexOf(key);
      if (index !== -1) {
        this.fileKeys.splice(index, 1);
      }
    }
    try {
      const path = this.getPath(key);
      unlinkSync(path);
    } catch (err) {
    }
  }
  clear() {
    this.fileKeys = [];
    this.store = {};
    rmSync(this.dirname, { recursive: true, force: true });
  }
  allKeys() {
    if (this.fileKeys == null) {
      try {
        this.fileKeys = readdirSync(this.dirname, { withFileTypes: true }).filter((item) => !item.isDirectory() && item.name.endsWith(this.extension)).map((item) => item.name.slice(0, -this.extensionLength)) || [];
      } catch (err) {
      }
    }
    let keys = [...this.fileKeys || []];
    for (let key of Object.keys(this.store)) {
      if (!keys.includes(key)) {
        keys.push(key);
      }
    }
    keys.sort();
    return keys;
  }
};
__name(FileStorage, "FileStorage");

// src/node/log-file.ts
import { createWriteStream, mkdirSync as mkdirSync2 } from "fs";
import { dirname as dirname2, resolve as resolve3 } from "path";
var namespaces = {};
function LoggerFileHandler(path, opt = {}) {
  const { level = -1 /* all */, filter = "*" } = opt;
  path = resolve3(process.cwd(), path);
  mkdirSync2(dirname2(path), { recursive: true });
  var stream = createWriteStream(path, { flags: "a" });
  const matchesNamespace = useNamespaceFilter(filter);
  const matchesLevel = useLevelFilter(level);
  return (msg) => {
    if (!matchesLevel(msg.level))
      return;
    if (!matchesNamespace(msg.name))
      return;
    const time2 = new Date().toISOString();
    let name = msg.name || "";
    let ninfo = namespaces[name || ""];
    if (ninfo == null) {
      namespaces[name] = ninfo;
    }
    let args = [
      `[${name || "*"}]`,
      renderMessages(msg.messages, { pretty: false })
    ];
    function write(...args2) {
      stream.write(args2.join("	") + "\n");
    }
    __name(write, "write");
    switch (msg.level) {
      case 1 /* info */:
        write(time2, `I|*  `, ...args);
        break;
      case 2 /* warn */:
        write(time2, `W|** `, ...args);
        break;
      case 3 /* error */:
        write(time2, `E|***`, ...args);
        break;
      default:
        write(time2, `D|   `, ...args);
        break;
    }
  };
}
__name(LoggerFileHandler, "LoggerFileHandler");

// src/node/log-util.ts
import { resolve as resolve4 } from "path";
function getStackLlocationList(stack) {
  var _a2, _b2;
  if (typeof stack !== "string")
    return [];
  return ((_b2 = (_a2 = stack == null ? void 0 : stack.split("\n")) == null ? void 0 : _a2.map((rawLine) => {
    let m = rawLine.match(/^\s+at.*(\((.*)\)|file:\/\/(.*)$)/);
    if (m) {
      let line = m[3] || m[2];
      if (line.endsWith(")"))
        line = line.slice(0, -1);
      return line;
    }
  })) == null ? void 0 : _b2.filter((v) => v != null)) || [];
}
__name(getStackLlocationList, "getStackLlocationList");
var cwd = resolve4(process.cwd());
var _a, _b;
var home = ((_a = process.env) == null ? void 0 : _a.HOME) ? resolve4((_b = process.env) == null ? void 0 : _b.HOME) : "";
function pathStripCwd(path) {
  if (path.includes("/node_modules/")) {
    return "";
  }
  const fileURL = "file://";
  if (path.startsWith(fileURL)) {
    return path.substr(fileURL.length);
  }
  if (cwd && path.startsWith(cwd)) {
    return path.substr(cwd.length + 1);
  }
  if (home && path.startsWith(home)) {
    path = "~/" + path.substr(home.length + 1);
  }
  return path;
}
__name(pathStripCwd, "pathStripCwd");
function extractFileInfo(stackLine) {
  let m = stackLine.match(/^\s*at.*(\((.*)\)|file:\/\/(.*)$)/);
  if (m) {
    let line = m[3] || m[2];
    if (line.endsWith(")"))
      line = line.slice(0, -1);
    return line;
  }
  return "";
}
__name(extractFileInfo, "extractFileInfo");
function getSourceLocation(level = 2, stripCwd = true) {
  var _a2;
  let stack = new Error().stack || "";
  let line = (_a2 = getStackLlocationList(stack)) == null ? void 0 : _a2[level];
  if (line && stripCwd) {
    line = pathStripCwd(line);
  }
  return line || "";
}
__name(getSourceLocation, "getSourceLocation");
function getStack() {
  return new Error().stack || "";
}
__name(getStack, "getStack");
function getSourceLocationByPrecedingPattern(patterns, stripCwd = true) {
  let line = "";
  let stack = new Error().stack || "";
  if (typeof stack === "string") {
    const lines = stack.split("\n").map((l) => l.trim());
    const index = lines.findIndex((l) => patterns.some((p) => l.startsWith(p)));
    line = lines[index + 1];
    if (line) {
      line = extractFileInfo(line);
    }
    if (line && stripCwd) {
      line = pathStripCwd(line);
    }
  }
  return line;
}
__name(getSourceLocationByPrecedingPattern, "getSourceLocationByPrecedingPattern");

// src/node/log-node.ts
import tty from "tty";
function isTTY() {
  try {
    return tty.isatty(process.stdout.fd);
  } catch (err) {
  }
  return false;
}
__name(isTTY, "isTTY");
var colors = [6, 2, 3, 4, 5, 1];
function selectColor(namespace) {
  let hash = 0;
  for (let i = 0; i < namespace.length; i++) {
    hash = (hash << 5) - hash + namespace.charCodeAt(i);
    hash |= 0;
  }
  return colors[Math.abs(hash) % colors.length];
}
__name(selectColor, "selectColor");
var namespaces2 = {};
var time = getTimestamp();
var useColors = tty.isatty(process.stderr.fd);
function log3(...args) {
  return process.stderr.write(renderMessages(args) + "\n");
}
__name(log3, "log");
var TTY_STYLE = {
  BOLD: "\x1B[1m",
  UNBOLD: "\x1B[2m",
  RED: "\x1B[31m",
  GREEN: "\x1B[32m",
  BLUE: "\x1B[34m",
  PURPLE: "\x1B[35m",
  GRAY: "\x1B[37m",
  ORANGE: "\x1B[38;5;208m",
  UNCOLOR: "\x1B[0m"
};
var colorEnd = "\x1B[0m";
function colorString(value, colorCode) {
  const colorStart = colorCode === 8 /* ORANGE */ ? TTY_STYLE.ORANGE : "\x1B[3" + (colorCode < 8 ? colorCode : "8;5;" + colorCode) + "m";
  return `${colorStart}${value}${colorEnd}`;
}
__name(colorString, "colorString");
function colorStringList(list, style, bold = true) {
  return list.map((value) => {
    if (typeof value !== "string")
      return value;
    let start = style;
    let end = colorEnd;
    if (bold) {
      start = `${TTY_STYLE.BOLD}${start}`;
      end = `${end}${TTY_STYLE.BOLD}`;
    }
    return `${start}${value}${end}`;
  });
}
__name(colorStringList, "colorStringList");
var loggerStackTraceDebug = "loggerStackTraceDebug-7d38e5a9214b58d29734374cdb9521fd964d7485";
function LoggerNodeHandler(opt = {}) {
  const {
    level = void 0,
    filter = void 0,
    colors: colors2 = isTTY(),
    levelHelper = true,
    nameBrackets = true,
    padding = 0,
    fill = 0,
    stack = true
  } = opt;
  const matchesNamespace = useNamespaceFilter(filter);
  const matchesLevel = useLevelFilter(level);
  return (msg) => {
    var _a2;
    if (!matchesLevel(msg.level))
      return;
    if (!matchesNamespace(msg.name))
      return;
    const timeNow = getTimestamp();
    let name = msg.name || "";
    let ninfo = namespaces2[name || ""];
    if (ninfo == null) {
      ninfo = {
        color: selectColor(name)
      };
      namespaces2[name] = ninfo;
    }
    const diff = formatMilliseconds(timeNow - time);
    let args;
    let displayName = nameBrackets ? `[${name}]` : name;
    if (padding > 0) {
      displayName = displayName.padStart(padding, " ");
    }
    if (fill > 0) {
      displayName = displayName.padEnd(fill, " ");
    }
    if (colors2 && useColors) {
      const c = ninfo.color;
      args = [colorString(displayName, c) + ` | `];
      if (msg.level === 2 /* warn */) {
        args.push(...colorStringList(msg.messages, TTY_STYLE.ORANGE));
      } else if (msg.level === 3 /* error */) {
        args.push(...colorStringList(msg.messages, TTY_STYLE.RED));
      } else {
        args.push(...msg.messages);
      }
      args.push(colorString(`+${diff}`, c));
    } else {
      args = [displayName, ...msg.messages];
      args.push(`+${diff}`);
    }
    if (((_a2 = msg.messages) == null ? void 0 : _a2[0]) === loggerStackTraceDebug) {
      console.log(getStack());
    }
    if (stack) {
      let line = "";
      if (typeof stack === "boolean") {
        line = getSourceLocationByPrecedingPattern(["at Function.", "at null.log (", "at log ("], true);
        if (!line) {
          line = getSourceLocation(0, true);
        }
      } else {
        const depth = typeof stack === "number" ? stack : 3;
        line = getSourceLocation(depth, true);
      }
      if (line) {
        args.push(colorString(`(${line})`, 7 /* GRAY */));
      }
    }
    switch (msg.level) {
      case 1 /* info */:
        if (levelHelper)
          args[0] = `I|*   ` + args[0];
        log3(...args);
        break;
      case 2 /* warn */:
        if (levelHelper)
          args[0] = (colors2 && useColors ? colorString(`W|**  `, 8 /* ORANGE */) : `W|**  `) + args[0];
        log3(...args);
        break;
      case 3 /* error */:
        if (levelHelper)
          args[0] = (colors2 && useColors ? colorString(`E|*** `, 1 /* RED */) : `E|*** `) + args[0];
        log3(...args);
        break;
      default:
        if (levelHelper)
          args[0] = `D|    ` + args[0];
        log3(...args);
        break;
    }
  };
}
__name(LoggerNodeHandler, "LoggerNodeHandler");

// src/node/crypto.ts
import nodeCrypto from "crypto";
try {
  if (nodeCrypto && nodeCrypto.webcrypto && typeof globalThis !== "undefined" && typeof globalThis.crypto === "undefined") {
    globalThis.crypto = nodeCrypto.webcrypto;
  }
} catch (err) {
  console.warn("Failed to polyfill webcrypto", err);
}

// src/node/log-context-node.ts
function setupLogContextNode() {
  var _a2;
  let handlers = [
    LoggerNodeHandler({
      padding: 32,
      nameBrackets: false
    })
  ];
  let logFilePath = (_a2 = process.env.ZEED_LOG) != null ? _a2 : process.env.LOG;
  if (logFilePath) {
    handlers.unshift(LoggerFileHandler(toPath(logFilePath)));
  }
  Logger.setHandlers(handlers);
}
__name(setupLogContextNode, "setupLogContextNode");
setupLogContextNode();

export {
  pbcopy,
  stringToPath,
  valueToPath,
  toPath,
  setupEnv,
  FileStorage,
  LoggerFileHandler,
  getStackLlocationList,
  getSourceLocation,
  getStack,
  getSourceLocationByPrecedingPattern,
  isTTY,
  colorString,
  colorStringList,
  loggerStackTraceDebug,
  LoggerNodeHandler
};
//# sourceMappingURL=chunk-4I4NSJEE.js.map