import { g as LogHandlerOptions, d as LogHandler, a as LogLevel } from './log-b80164f3';

declare type BinInput = Uint8Array | ArrayBuffer | string | number[];
declare function _encodeUtf8Polyfill(str: string): Uint8Array;
declare function _decodeUtf8Polyfill(buf: Uint8Array): string;
declare function stringToUInt8Array(text: string): Uint8Array;
declare function Uint8ArrayToString(bin: Uint8Array): string;
declare function toUint8Array(data: BinInput): Uint8Array;
declare function toHex(bin: BinInput): string;
declare function toBase64(bin: BinInput): string;
declare function toBase64Url(bin: BinInput): string;
declare function fromBase64(s: string): Uint8Array;
/** Compare contents of binary arrays */
declare function equalBinary(a: ArrayBuffer | Uint8Array, b: ArrayBuffer | Uint8Array): boolean;
declare function jsonToUint8Array(json: any): Uint8Array | never;
declare function Uint8ArrayToJson<T = any>(data: Uint8Array): T | never;

declare function randomUint8Array(length?: number): Uint8Array;
declare function digest(message: BinInput, algorithm?: AlgorithmIdentifier): Promise<Uint8Array>;
declare function deriveKeyPbkdf2(secret: BinInput, opt?: {
    iterations?: number;
    salt?: BinInput;
}): Promise<CryptoKey>;
declare function encrypt(data: Uint8Array, key: CryptoKey): Promise<Uint8Array>;
declare function decrypt(data: Uint8Array, key: CryptoKey): Promise<Uint8Array>;

declare function csv(data: any[], headerRow: string[]): string;

declare type Json = string | number | boolean | null | {
    [property: string]: Json;
} | Json[];
interface ObjectStorage<T = any> {
    setItem(key: string, value: T): void;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}
declare type NestedArray<T> = T | Array<NestedArray<T>>;

declare function arrayUnique<T>(x: T[]): T[];
declare function arrayMinus<T>(x: T[], y: T[]): T[];
declare function arrayUnion<T>(...a: T[][]): T[];
/** `[1,[2,3]]` becomes `[1,2,3]` */
declare function arrayFlatten<T>(...list: NestedArray<T>[]): T[];
declare function arrayIntersection<T>(x: T[], y: T[]): T[];
declare function arraySymmetricDifference<T>(x: T[], y: T[]): T[];
declare function arrayRemoveElement<T>(arr: T[], el: T): T[];
/** Only have it once in the set */
declare function arraySetElement<T>(arr: T[], el: T): T[];
declare function arrayFilterInPlace<T>(array: T[], fn: (el: T) => boolean): T[];
declare function arrayToggleInPlace<T>(array: T[], el: T): T[];
declare function arrayEmptyInPlace<T>(array: T[]): T[];
declare function arraySorted<T>(arr: Iterable<T> | ArrayLike<T>, cond?: ((a: T, b: T) => number) | undefined): T[];
declare function arraySortedNumbers(arr: number[]): number[];
declare function arrayIsEqual<T>(array1: T[], array2: T[]): boolean;
declare function arrayShuffleInPlace<T>(array: T[]): T[];
declare function arrayShuffle<T>(array: T[]): T[];
/** Randomly shuffle the order of the array's elements. Force to have a different order if array has more than one element. */
declare function arrayShuffleForce<T>(array: T[]): T[];
declare function arrayRandomElement<T>(array: T[]): T;
declare function arrayMax<T>(...array: NestedArray<T>[]): T;
declare function arrayMin<T>(...array: NestedArray<T>[]): T;
declare function arraySum(...array: NestedArray<number>[]): number;
declare function arrayAvg(...array: NestedArray<number>[]): number;
declare function createArray<T>(size?: number, item?: T | ((index: number) => T)): T[];

declare function useBase(alphaOrBase: string | number): {
    encode: (source: BinInput, padToLength?: number) => string;
    decode: (source: string, padToLength?: number) => Uint8Array;
};
declare const encodeBase16: (source: BinInput, padToLength?: number) => string;
declare const decodeBase16: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase32: (source: BinInput, padToLength?: number) => string;
declare const decodeBase32: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase58: (source: BinInput, padToLength?: number) => string;
declare const decodeBase58: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase62: (source: BinInput, padToLength?: number) => string;
declare const decodeBase62: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase64: (source: BinInput, padToLength?: number) => string;
declare const decodeBase64: (source: string, padToLength?: number) => Uint8Array;

declare const toCamelCase: (s: string) => string;
declare function toCapitalize(s: string): string;
declare function toCapitalizeWords(s: string): string;
declare function fromCamelCase(str: string, separator?: string): string;

declare function stringToBoolean(value?: string, defaultValue?: boolean): boolean;
declare function stringToInteger(value?: string, defaultValue?: number): number;
declare function stringToFloat(value?: string, defaultValue?: number): number;
declare function valueToBoolean(value?: any, defaultValue?: boolean): boolean;
declare function valueToInteger(value?: any, defaultValue?: number): number;
declare function valueToFloat(value?: any, defaultValue?: number): number;
declare function valueToString(value?: any, defaultValue?: string): string;
declare const toFloat: typeof valueToFloat;
declare const toInt: typeof valueToInteger;
declare const toString: typeof valueToString;
declare const toBool: typeof valueToBoolean;
declare type RenderMessagesOptions = {
    trace?: boolean;
    pretty?: boolean;
};
declare function formatMessages(messages: any[], opt?: RenderMessagesOptions): any[];
declare function renderMessages(messages: any[], opt?: RenderMessagesOptions): string;
declare function fixBrokenUth8String(brokenString: string): string;

declare type CurrencyInput = number | string | Currency;
declare type CurrencyFormat = (currency?: Currency, opts?: CurrencyOptions) => string;
interface CurrencyOptions {
    symbol?: string;
    separator?: string;
    decimal?: string;
    errorOnInvalid?: boolean;
    precision?: number;
    increment?: number;
    useVedic?: boolean;
    pattern?: string;
    negativePattern?: string;
    format?: CurrencyFormat;
    fromCents?: boolean;
    groups?: RegExp;
}
declare function currency(value: CurrencyInput, opts?: CurrencyOptions): Currency;
/**
 * Immuteable currency representation
 */
declare class Currency {
    readonly intValue: number;
    readonly value: number;
    private readonly _settings;
    private readonly _precision;
    constructor(value: CurrencyInput, opts?: CurrencyOptions);
    add(number: CurrencyInput): Currency;
    subtract(number: CurrencyInput): Currency;
    multiply(number: CurrencyInput): Currency;
    divide(number: CurrencyInput): Currency;
    distribute(count: number): number[];
    dollars(): number;
    cents(): number;
    format(options: CurrencyOptions | Function): any;
    toString(): string;
    toJSON(): number;
    static zero: Currency;
    static one: Currency;
    static hundred: Currency;
    static sum(...array: (CurrencyInput | CurrencyInput[])[]): Currency;
    static avg(...array: (CurrencyInput | CurrencyInput[])[]): Currency;
}
declare type RoundingMode = (value: number) => number;
declare const isHalf: (value: number) => boolean;
declare const isEven: (value: number) => boolean;
declare const roundUp: RoundingMode;
declare const roundDown: RoundingMode;
declare const roundHalfUp: RoundingMode;
/**
 * Round a number with half values to nearest odd integer.
 */
declare const roundHalfOdd: RoundingMode;
/**
 * Round a number with half values to nearest integer farthest from zero.
 */
declare const roundHalfAwayFromZero: RoundingMode;
/**
 * Round a number with half values down.
 */
declare const roundHalfDown: RoundingMode;
/**
 * Round a number with half values to nearest even integer.
 * https://wiki.c2.com/?BankersRounding
 */
declare const roundHalfEven: RoundingMode;
/**
 * Round a number with half values to nearest integer closest to zero.
 */
declare const roundHalfTowardsZero: RoundingMode;

declare const DAY_MS: number;
declare type DayInput = number | string | Date | Day | [number, number, number];
/** Date represented as positive integer value. Years smaller 0 are not supported.  */
declare class Day {
    days: number;
    constructor(days?: DayInput);
    static fromNumber(n: number): Day;
    static fromString(dateString: string): Day | undefined;
    static fromDate(date: Date, gmt?: boolean): Day;
    static fromDateGMT(date: Date): Day;
    static from(value: DayInput, gmt?: boolean): Day | undefined;
    toNumber(): number;
    /** Just for future extensions */
    toJson(): number;
    toString(sep?: string): string;
    toDate(gmt?: boolean): Date;
    toDateGMT(): Date;
    get year(): number;
    get month(): number;
    get day(): number;
    dayOffset(offset: number): Day;
    /** Very stupid approach, only works for days <= 28 */
    monthOffset(offset: number): Day;
    daysUntil(otherDay: DayInput): number;
    yesterday(): Day;
    tomorrow(): Day;
}
declare function forEachDay(from: DayInput, to: DayInput, handler: (date: Day) => Promise<void> | void): Promise<void>;
declare function today(): Day;
declare function day(days?: DayInput): Day;

declare function deepEqual(a: any, b: any, hash?: WeakSet<object>): boolean;
declare function deepMerge(target: any, ...sources: any[]): any;

declare const escapeHTML: (s: string) => string;
declare const unescapeHTML: (s: string) => string;

declare type Primitive = null | undefined | string | number | boolean | symbol | bigint;
declare function isObject(obj: unknown): obj is object;
declare function isPrimitive(obj: unknown): obj is Primitive;
declare function isArray(obj: unknown): obj is Array<any>;
declare function isRecord(obj: unknown): obj is Record<string, any>;
declare function isString(obj: unknown): obj is string;
declare function isNumber(obj: unknown): obj is number;
declare function isInteger(obj: unknown): obj is number;
declare function isSafeInteger(obj: unknown): obj is number;
declare function isBoolean(obj: unknown): obj is boolean;
declare function isNullOrUndefined(obj: unknown): obj is null | undefined;

declare type EntryProcessor = (key: string, value: any) => any;
/**
 * Similar to JSON.stringify but can handle circular references
 */
declare function jsonStringify(obj: any, replacer?: EntryProcessor | null, spaces?: string | number | null, cycleReplacer?: EntryProcessor): string;

declare function getSecureRandom(): number;
declare function getSecureRandomIfPossible(): number;
declare function randomBoolean(bias?: number): boolean;
/** max is not included, min is included */
declare function randomInt(max?: number, min?: number): number;
declare function randomFloat(max?: number, min?: number): number;
declare function between(min: number, value: number, max: number): number;
/** See also arraySum */
declare function sum(array: number[]): number;
/** See also arrayAvg */
declare function avg(array: number[]): number;

declare function parseOrderby(value?: string): {
    field: string;
    orderby: string;
    asc: boolean;
    desc: boolean;
};
declare function composeOrderby(field: string, asc?: boolean): string;
declare function cmp(a: any, b: any, asc?: boolean): number;
declare function sortedOrderby<T>(values: T[], ...orderby: string[]): T[];

declare function toValidFilename(string: string): string | never;

declare function escapeRegExp(value: RegExp | string): string;

interface SortableItem {
    sort_weight: number;
}
declare function startSortWeight(items: SortableItem[]): number;
declare function endSortWeight(items: SortableItem[]): number;
declare function moveSortWeight(newIndex: number, oldIndex: number, items: SortableItem[]): number;
declare function sortedItems<T extends SortableItem>(items: T[]): T[];

declare function linkifyPlainText(text: string): string;
declare function toHumanReadableUrl(url: string): string;
declare function encodeQuery(data: Record<string, any>): string;
declare function parseQuery(queryString: string): any;

/**
 * Call a create function if key does not yet exist on an object. Returns the found or created object. Example:
 *
 * ```js
 * function createRoom(room, rooms) { return new Room() }
 * ensureKey(rooms, room, createRoom).enter()
 * ```
 */
declare function ensureKey<T>(obj: Record<string, T>, key: string, createFn: (key?: string, obj?: Record<string, T>) => T): T;
/**
 * Call a create function if key does not yet exist on an object. Returns the found or created object. Example:
 *
 * ```js
 * async function fetchItem(id, cache) { ... }
 * let data = await ensureKey(cache, id, fetchItem)
 * ```
 */
declare function ensureKeyAsync<T>(obj: Record<string, T>, key: string, createFn: (key?: string, obj?: Record<string, T>) => Promise<T>): Promise<T>;
declare function size(obj: any): any;
declare function last<T>(array?: T[]): T | undefined;
declare function empty(value: any): boolean;
declare function cloneObject<T>(obj: T): T;
declare function cloneJsonObject<T = Json>(obj: T): T;
/** Cache result of a function. Same arguments have to always return the same result in order to get expected optimization! */
declare function memoize<In extends any, Out extends any>(fn: (arg: In) => Out): (arg: In) => Out;

declare function regExpString(rx: string | RegExp): string;
declare function regExpEscape(str: string): string;
declare class XRX {
    lookBehind: {
        rx: RegExp;
        expect: boolean;
    } | undefined;
    namedGroups: Record<string, number>;
    names: string[];
    rx: RegExp;
    constructor(pattern: string | RegExp, flags?: string);
    get lastIndex(): number;
    set lastIndex(index: number);
    _handleMatch(m: any): any;
    exec(str: string): any;
    execAll(str: string): any[];
    replace(str: string, replacement: string | Function): string;
}

declare type DisposerFunction = () => any | Promise<any>;
declare type Disposer = DisposerFunction | {
    dispose?: Function | Promise<unknown>;
    cleanup?: Function | Promise<unknown>;
};
interface Disposable {
    dispose(): void | Promise<void>;
}
interface UseDisposeConfig {
    name?: string;
}
declare function useDispose(config?: string | UseDisposeConfig): (() => Promise<void>) & {
    track: (obj: Disposer) => DisposerFunction;
    add: (obj: Disposer) => DisposerFunction;
    untrack: (disposable: Disposer) => Promise<void>;
    dispose: () => Promise<void>;
    exec: () => Promise<void>;
    getSize(): number;
};
declare type UseDispose = ReturnType<typeof useDispose>;
/** @deprecated use `useDispose` instead */
declare const useDisposer: typeof useDispose;
declare function useDefer(config?: {
    mode?: "lifo" | "fifo";
}): ((expectSync?: boolean) => Promise<void>) & {
    add: (obj: Disposer) => void;
    exec: (expectSync?: boolean) => Promise<void>;
    getSize(): number;
};
declare type UseDefer = ReturnType<typeof useDefer>;
declare function useTimeout(fn: Function, timeout?: number): DisposerFunction;
declare function useInterval(fn: Function, interval: number): DisposerFunction;
declare function useEventListener(emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]): DisposerFunction;

declare global {
    interface ZeedGlobalContext {
    }
}
declare function getGlobalContext(): ZeedGlobalContext;

declare function isLocalHost(hostname?: string): boolean;

/**
 * Very basic logger. Please take a look at the browser and node
 * optimized loggers. This one is just the absolute fallback option.
 *
 * @param level Log level
 * @returns Logger
 */
declare function LoggerConsoleHandler(opt?: LogHandlerOptions): LogHandler;

interface NamespaceFilter {
    (name: string): boolean;
    accept: RegExp[];
    reject: RegExp[];
    filter: string;
}
declare function getNamespaceFilterString(defaultNamespaceFilter: any): string;
/**
 * Filter as described here https://github.com/visionmedia/debug#wildcards
 *
 * @param filter Namespace filter
 * @returns Function to check if filter applies
 */
declare function useNamespaceFilter(filter?: string): NamespaceFilter;
declare function useLevelFilter(filter?: string | number): (level: LogLevel) => boolean;

declare type EmitterHandler = (...objs: any[]) => void;
declare type EmitterAllHandler<T = string> = (key: T, ...objs: any[]) => void;
declare type ListenerSignature<L> = {
    [E in keyof L]: (...args: any[]) => any;
};
declare type DefaultListener = {
    [k: string]: (...args: any[]) => any;
};
declare class Emitter<L extends ListenerSignature<L> = DefaultListener> implements Disposable {
    subscribers: any;
    subscribersOnAny: any[];
    call: L;
    emit<U extends keyof L>(event: U, ...args: Parameters<L[U]>): Promise<boolean>;
    onAny(fn: EmitterHandler): void;
    on<U extends keyof L>(event: U, listener: L[U]): DisposerFunction;
    onCall(handlers: Partial<L>): void;
    once<U extends keyof L>(event: U, listener: L[U]): DisposerFunction;
    off<U extends keyof L>(event: U, listener: L[U]): this;
    removeAllListeners(event?: keyof L): this;
    dispose(): void;
}
declare global {
    interface ZeedGlobalContext {
        emitter?: Emitter;
    }
    interface ZeedGlobalEmitter {
    }
}
/** Global emitter that will listen even across modules */
declare function getGlobalEmitter(): Emitter<ZeedGlobalEmitter>;
declare const messages: Emitter<DefaultListener>;
declare function lazyListener(emitter: any, listenerKey?: string): (key?: string, skipUnmatched?: boolean) => Promise<any>;

/** See http://developer.mozilla.org/en-US/docs/Web/API/MessageEvent */
interface ChannelMessageEvent {
    data: any;
    origin?: string;
    lastEventId?: string;
}
/**
 * Inspired by
 * http://developer.mozilla.org/en-US/docs/Web/API/BroadcastChannel
 * https://deno.com/deploy/docs/runtime-broadcast-channel
 * */
declare abstract class Channel extends Emitter<{
    message(event: ChannelMessageEvent): void;
    messageerror(event: ChannelMessageEvent): void;
    connect(): void;
    disconnect(): void;
    close(): void;
}> {
    id: string;
    abstract isConnected?: boolean;
    abstract postMessage(data: any): void;
    close(): void;
}
/** Very basic channel demonstrating local communication */
declare class LocalChannel extends Channel {
    isConnected: boolean;
    other?: LocalChannel;
    postMessage(data: any): void;
}
declare function fakeWorkerPair(): [LocalChannel, LocalChannel];

interface Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class NoopEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class JsonEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class CryptoEncoder implements Encoder {
    key: CryptoKey;
    constructor(key: CryptoKey);
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}

declare type MessageAction = {
    name: string;
    id: string;
    args?: Json[];
};
declare type MessageResult = {
    id: string;
    result?: Json;
    error?: {
        stack?: string;
        name: string;
        message: string;
    };
};
declare type Message = MessageAction | MessageResult;
declare type MessagesOptions = {
    timeout?: number;
};
declare type MessagesDefaultMethods<L> = {
    dispose(): void;
    connect?(channel: Channel): void;
    options(opt: MessagesOptions): L;
};
declare type MessagesMethods<L> = L & MessagesDefaultMethods<L>;
declare type MessageDefinitions = Record<any, (...args: any) => Promise<any>>;
declare type MessageHub = {
    dispose(): void;
    connect: (newChannel: Channel) => void;
    listen<L extends MessageDefinitions>(newHandlers: L): void;
    send<L extends MessageDefinitions>(): MessagesMethods<L>;
};
declare const createPromiseProxy: <P extends object>(fn: (name: string, args: any[], opt: any) => Promise<unknown>, opt: MessagesOptions, predefinedMethods?: any) => P;
declare function useMessageHub(opt?: {
    name?: string;
    channel?: Channel;
    encoder?: Encoder;
    retryAfter?: number;
    ignoreUnhandled?: boolean;
    debug?: boolean;
}): MessageHub;

interface PubSubConfig {
    channel: Channel;
    encoder?: Encoder;
    name?: string;
    debug?: boolean;
}
declare class PubSub<L extends ListenerSignature<L> = DefaultListener> extends Emitter<L> {
    name: string;
    channel: Channel;
    encoder: Encoder;
    log: any;
    debug: boolean;
    get shortId(): string;
    constructor(opt: PubSubConfig);
    private emitSuper;
    emit<U extends keyof L>(event: U, ...args: Parameters<L[U]>): Promise<boolean>;
    publish: <U extends keyof L>(event: U, ...args: Parameters<L[U]>) => Promise<boolean>;
    subscribe: <U extends keyof L>(event: U, listener: L[U]) => DisposerFunction;
}
declare function usePubSub<L extends ListenerSignature<L> = DefaultListener>(opt: PubSubConfig): PubSub<L>;

declare type ArgumentsType<T> = T extends (...args: infer A) => any ? A : never;
declare type ReturnType$1<T> = T extends (...args: any) => infer R ? R : never;
interface RPCOptions<Remote> {
    /** No return values expected */
    onlyEvents?: boolean;
    /** Names of remote functions that do not need response. */
    eventNames?: (keyof Remote)[];
    /** Function to post raw message */
    post: (data: any) => void;
    /** Listener to receive raw message */
    on: (fn: (data: any) => void) => void;
    /** Custom function to serialize data */
    serialize?: (data: any) => any;
    /** Custom function to deserialize data */
    deserialize?: (data: any) => any;
}
declare type RPCFn<T> = {
    /** Call the remote function and wait for the result. */
    (...args: ArgumentsType<T>): Promise<Awaited<ReturnType$1<T>>>;
    /** Send event without asking for response */
    asEvent(...args: ArgumentsType<T>): void;
};
declare type RPCReturn<RemoteFunctions> = {
    [K in keyof RemoteFunctions]: RPCFn<RemoteFunctions[K]>;
};
declare function useRPC<RemoteFunctions = {}, LocalFunctions = {}>(functions: LocalFunctions, options: RPCOptions<RemoteFunctions>): RPCReturn<RemoteFunctions>;

declare type Mutex = (fn: Function, elseFn?: Function) => boolean;
declare type AsyncMutex = (fn: Function, elseFn?: Function) => Promise<boolean>;
declare function useMutex(): Mutex;
declare function useAsyncMutex(): AsyncMutex;

declare type httpMethod = "GET" | "POST" | "PUT" | "DELETE" | "HEAD" | "CONNECT" | "OPTIONS" | "TRACE" | "PATCH";
declare function fetchBasic(url: string, fetchOptions?: any, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<Response | undefined>;
declare function fetchJson<T = Json>(url: string, opts?: any, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<T | undefined>;
declare function fetchOptionsFormURLEncoded(data: Object, method?: httpMethod): {
    headers: {
        "Content-Type": string;
    };
    body: string;
    cache: string;
    redirect: string;
    method: httpMethod;
};
declare function fetchOptionsJson(data: Object, method?: httpMethod): {
    headers: {
        "Content-Type": string;
        Accept: string;
    };
    body: string;
    cache: string;
    redirect: string;
    method: httpMethod;
};
declare function fetchText(url: string, opts?: any, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<string | undefined>;

declare function getWindow(): any | undefined;
declare function getNavigator(): any | undefined;
declare function getGlobal(): any;
declare function detect(info?: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
}): {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
declare const isBrowser: () => boolean;
declare const platform: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
/**
 * Before closing the tab/window or quitting the node process
 * allow to do something important first
 */
declare function useExitHandler(handler: () => void): void;

interface PoolConfig {
    maxParallel?: number;
}
declare function usePool<T = any>(config?: PoolConfig): {
    cancel: (id: string) => void;
    enqueue(id: string, task: QueueTask<T>): () => void;
};
declare type Pool = ReturnType<typeof usePool>;

/** Sleep for `milliSeconds`. Example 1s: `await sleep(1000)` */
declare function sleep(milliSeconds: number): Promise<void>;
declare function immediate(): Promise<void>;
declare const timeoutReached: unique symbol;
declare function timeout<T>(promise: Promise<T>, milliSeconds: number, timeoutValue?: symbol): Promise<T | typeof timeoutValue>;
declare const timoutError: Error;
declare function isTimeout(value: any): boolean;
declare function tryTimeout<T>(promise: Promise<T>, milliSeconds: number): Promise<T | undefined>;
/** Wait for `event` on `obj` to emit. Resolve with result or reject on `timeout` */
declare function waitOn(obj: any, event: string, timeoutMS?: number): Promise<any>;
declare function isPromise<T>(value: Promise<T> | T): value is Promise<T>;
/** This is exactly what Prose.resolve(x) is supposed to be: return a Promise no matter what type x is */
declare function promisify<T>(value: Promise<T> | T): Promise<T>;

declare type QueueTask<T = any> = () => Promise<T>;
/** Guarentee serial execution of tasks. Able to wait, pause, resume and cancel all. */
declare class SerialQueue {
    private queue;
    private isPaused;
    private waitToFinish;
    private currentTask?;
    private log;
    name: string;
    constructor(opt?: {
        name?: string;
        logLevel?: LogLevel;
    });
    private performNext;
    /** Enqueue task to be executed when all other tasks are done. Except `immediate = true`. */
    enqueue<T>(task: QueueTask<T>, opt?: {
        immediate?: boolean;
        name?: string;
    }): Promise<T>;
    /** If a task is already performing, execute immediately. Otherwise enqueue as usual. */
    enqueueReentrant<T>(task: QueueTask<T>, opt?: {
        name?: string;
    }): Promise<T>;
    /** Remove all tasks from queue that are not yet executing. */
    cancelAll(unblock?: boolean): Promise<void>;
    /** Pause execution after current task is finished. */
    pause(): Promise<void>;
    /** Resume paused queue. */
    resume(): void;
    /** Wait for all tasks to finish */
    wait(): Promise<unknown>;
}

interface MemStorageOptions {
    objectFromString?: (data: string) => any;
    objectToString?: (data: any) => string;
}
declare class MemStorage<T = Json> implements ObjectStorage<T> {
    private store;
    constructor(opt?: MemStorageOptions);
    setItem(key: string, value: T): void;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}

interface ThrottleOptions {
    delay?: number;
    trailing?: boolean;
    leading?: boolean;
}
declare type ThrottleFunction = Function & {
    cancel: () => void;
    dispose: () => void;
};
interface DebounceOptions {
    delay?: number;
}
declare type DebounceFunction = ThrottleFunction;
/**
 * A special throttle implementation that tries to distribute execution
 * in an optimal way.
 *
 * For UI usage the function is executed on first occasion (`leading`).
 * If more calls follow it will again be executed at end (`trailing`).
 * If the next call is inside the timeframe, it is delayed until `trailing`.
 * This avoids timewise too close calls.
 * It is possible to `cancel` the timeout and to `flush` a call, e.g. if
 * leaving UI situation where a final call is required to write data or similar.
 */
declare function throttle(callback: Function, opt?: ThrottleOptions): ThrottleFunction;
/**
 * Debounce fits best for filtering a large peak of events.
 * For UI event filtering throttle is probably a better choice.
 */
declare function debounce(callback: Function, opt?: DebounceOptions): DebounceFunction;

/**
 * @returns Timestamp in miliseconds
 */
declare const getTimestamp: () => number;
declare function formatMilliseconds(ms: number): string;
declare function parseDate(...dateCandidates: (string | Date)[]): Date | undefined;

declare function uuidBytes(): Uint8Array;
declare const uuid32bit: () => number;
declare function uuid(): string;
declare function uuidEncode(bytes: Uint8Array): string;
declare function uuidDecode(uuid: string): Uint8Array;
declare function uuidB32(): string;
declare function uname(name?: string): string;
declare function qid(): string;
declare const uuidv4: () => string;
declare function suidBytes(): Uint8Array;
declare function suid(): string;
declare function suidDate(id: string): Date;
declare function suidBytesDate(id: Uint8Array): Date;

export { toCamelCase as $, arraySum as A, arrayAvg as B, createArray as C, useBase as D, encodeBase16 as E, decodeBase16 as F, encodeBase32 as G, decodeBase32 as H, encodeBase58 as I, decodeBase58 as J, encodeBase62 as K, decodeBase62 as L, encodeBase64 as M, decodeBase64 as N, BinInput as O, _decodeUtf8Polyfill as P, stringToUInt8Array as Q, toUint8Array as R, toHex as S, toBase64 as T, Uint8ArrayToString as U, toBase64Url as V, fromBase64 as W, equalBinary as X, jsonToUint8Array as Y, Uint8ArrayToJson as Z, _encodeUtf8Polyfill as _, deriveKeyPbkdf2 as a, composeOrderby as a$, toCapitalize as a0, toCapitalizeWords as a1, fromCamelCase as a2, stringToBoolean as a3, stringToInteger as a4, stringToFloat as a5, valueToBoolean as a6, valueToInteger as a7, valueToFloat as a8, valueToString as a9, today as aA, day as aB, deepEqual as aC, deepMerge as aD, escapeHTML as aE, unescapeHTML as aF, Primitive as aG, isObject as aH, isPrimitive as aI, isArray as aJ, isRecord as aK, isString as aL, isNumber as aM, isInteger as aN, isSafeInteger as aO, isBoolean as aP, isNullOrUndefined as aQ, jsonStringify as aR, getSecureRandom as aS, getSecureRandomIfPossible as aT, randomBoolean as aU, randomInt as aV, randomFloat as aW, between as aX, sum as aY, avg as aZ, parseOrderby as a_, toFloat as aa, toInt as ab, toString as ac, toBool as ad, RenderMessagesOptions as ae, formatMessages as af, renderMessages as ag, fixBrokenUth8String as ah, CurrencyInput as ai, currency as aj, Currency as ak, RoundingMode as al, isHalf as am, isEven as an, roundUp as ao, roundDown as ap, roundHalfUp as aq, roundHalfOdd as ar, roundHalfAwayFromZero as as, roundHalfDown as at, roundHalfEven as au, roundHalfTowardsZero as av, DAY_MS as aw, DayInput as ax, Day as ay, forEachDay as az, decrypt as b, MessageDefinitions as b$, cmp as b0, sortedOrderby as b1, toValidFilename as b2, escapeRegExp as b3, SortableItem as b4, startSortWeight as b5, endSortWeight as b6, moveSortWeight as b7, sortedItems as b8, linkifyPlainText as b9, isLocalHost as bA, LoggerConsoleHandler as bB, getNamespaceFilterString as bC, useNamespaceFilter as bD, useLevelFilter as bE, ChannelMessageEvent as bF, Channel as bG, LocalChannel as bH, fakeWorkerPair as bI, EmitterHandler as bJ, EmitterAllHandler as bK, ListenerSignature as bL, DefaultListener as bM, Emitter as bN, getGlobalEmitter as bO, messages as bP, lazyListener as bQ, Encoder as bR, NoopEncoder as bS, JsonEncoder as bT, CryptoEncoder as bU, MessageAction as bV, MessageResult as bW, Message as bX, MessagesOptions as bY, MessagesDefaultMethods as bZ, MessagesMethods as b_, toHumanReadableUrl as ba, encodeQuery as bb, parseQuery as bc, ensureKey as bd, ensureKeyAsync as be, size as bf, last as bg, empty as bh, cloneObject as bi, cloneJsonObject as bj, memoize as bk, regExpString as bl, regExpEscape as bm, XRX as bn, DisposerFunction as bo, Disposer as bp, Disposable as bq, useDispose as br, UseDispose as bs, useDisposer as bt, useDefer as bu, UseDefer as bv, useTimeout as bw, useInterval as bx, useEventListener as by, getGlobalContext as bz, csv as c, suidDate as c$, MessageHub as c0, createPromiseProxy as c1, useMessageHub as c2, PubSub as c3, usePubSub as c4, ArgumentsType as c5, ReturnType$1 as c6, RPCOptions as c7, RPCFn as c8, RPCReturn as c9, tryTimeout as cA, waitOn as cB, isPromise as cC, promisify as cD, QueueTask as cE, SerialQueue as cF, MemStorageOptions as cG, MemStorage as cH, throttle as cI, debounce as cJ, getTimestamp as cK, formatMilliseconds as cL, parseDate as cM, Json as cN, ObjectStorage as cO, NestedArray as cP, uuidBytes as cQ, uuid32bit as cR, uuid as cS, uuidEncode as cT, uuidDecode as cU, uuidB32 as cV, uname as cW, qid as cX, uuidv4 as cY, suidBytes as cZ, suid as c_, useRPC as ca, Mutex as cb, AsyncMutex as cc, useMutex as cd, useAsyncMutex as ce, httpMethod as cf, fetchBasic as cg, fetchJson as ch, fetchOptionsFormURLEncoded as ci, fetchOptionsJson as cj, fetchText as ck, getWindow as cl, getNavigator as cm, getGlobal as cn, detect as co, isBrowser as cp, platform as cq, useExitHandler as cr, usePool as cs, Pool as ct, sleep as cu, immediate as cv, timeoutReached as cw, timeout as cx, timoutError as cy, isTimeout as cz, digest as d, suidBytesDate as d0, encrypt as e, arrayUnique as f, arrayMinus as g, arrayUnion as h, arrayFlatten as i, arrayIntersection as j, arraySymmetricDifference as k, arrayRemoveElement as l, arraySetElement as m, arrayFilterInPlace as n, arrayToggleInPlace as o, arrayEmptyInPlace as p, arraySorted as q, randomUint8Array as r, arraySortedNumbers as s, arrayIsEqual as t, arrayShuffleInPlace as u, arrayShuffle as v, arrayShuffleForce as w, arrayRandomElement as x, arrayMax as y, arrayMin as z };
